/*  COMPILE INSTRUCTIONS FOR TURBO C++ 1.0
Memory model: large (all models with far code (i.e. all except tiny and small)
                    should work)
Project creating: 1)Project/Open project
                  2)Project/Add item/Keytest.cpp
                  3)Project/Add item/kbhandl.asm
                  4)Select kbhandl.asm in project window
                  5)Project/local options
                  6)Set translator to Turbo Assembler

If you assemble the file separately then make sure that you turn
case-sensitive link for global symbols on.

*/

/*                    PROGRAM DESCRIPTION
This program, in conjunction with kbhandl.asm, implements a keyboard handler
that keeps track of which keys are pressed.
Press ESC to stop the demo.

InstallKeyboardHandler(PSPAddress): installs the new handler. The argument
       that must be passed is the address of the program segment prefix (PSP)
       In Turbo C++ 1.0 you get it with the function getpid().
       InstallKeyboardHandler automatically sets up an exit procedure that
       makes sure that the handler is deinstalled properly, EVEN IF the
       program is aborted by some way that bypasses atexit() functions.
DeinstallKeyboardHandler(): usually doesn't have to be called manually. It
                           restores the original keyboard handler.
Int9PassThrough: If set to any value !=0 (default) the keyboard map works
                 transparent, i.e. the keypresses get passed through to
                 the original int9 handler. This is useful since it
                 makes it possible to use the standard functions like
                 kbhit(), getch(), etc.
                 If set to 0, then the keypresses don't get passed through.
                 This mode is useful in situations where you use only the
                 keyboard map but don't want to read characters via
                 standard functions.
                 NOTE: If you don't set Int9PassThrough to 0, yet don't call
                 getch() or any other function that reads from the keyboard,
                 the keyboard buffer will overrun, which will cause the
                 old int9 handler to beep on each further keypress.
                 Clearing the keyboard buffer can be
                 done via "while (kbhit()) getch();"
KeyMap: is an array of 256 unsigned chars. 1 means the respective key is
        being pressed. 0 means it is not. Keys with make codes that don't
        start with E0, map to the entry that corresponds to the make code.
        Keys with make codes that start with E0 map to the entry
        (make code+128). Don't get confused. The E0 is also counted as a
        key. It maps to entry 0x60, so this entry will always be 1 when
        an arrow key is pressed.

*/

/*       FURTHER NOTES:
1)Debugging programs that use interrupts is a nasty thing. You must always
 make sure that the interrupt gets deinstalled. Note that the exit procedure
 that is installed by the new keyboard handler DOES NOT get called when you
 interrupt a debugging session by restarting the program before it
 has finished! Before you restart the program you must make
 sure that it is either terminated correctly via exit(), abort(),... or that
 DeinstallKeyboardHandler() has been called.
2)Don't be confused if keyboard input behaves strangely after the program
  has been executed from your IDE.
  If the Int9PassThrough=0 command has been executed
  before you released the key you used to start the program, the keyboard
  driver doesn't notice Ctrl-, Alt-,... has been released. Simply press and
  release the key again.
  Example: You start the program from inside your IDE via Ctrl-F9. You are
  so slow at releasing the Ctrl-key that the program has already started and
  executed the Int9PassThrough=0 before you release the key. The old keyboard
  driver now doesn't notice that you release the Ctrl-key. After the program
  has finished, your IDE behaves as if the Ctrl-key is still being pressed.
  Simply press and release the Ctrl-key again, and everything will be fine.
3)The Pause-key doesn't show up in the keymap. This is not a bug in
  the keyboard handler, but a hardware problem. The Pause-key is the only
  key on the keyboard that sends its break code immediately after the make
  code. This means that you could only detect the pressing of the Pause key,
  but not the releasing, which means it cannot be used with a keymap.
4)Don't be confused if the Print Screen key still works. This usually means
  that you work inside a DOS box of a protected mode OS. This OS always gets
  keyboard input, no matter whether Int9PassThrough is 0 or not.
*/

#include <dos.h>
#include <process.h>

extern "C" int InstallKeyboardHandler(unsigned int PSPAddr);
extern "C" int DeinstallKeyboardHandler();
extern unsigned char KeyMap[256];
extern unsigned int Int9PassThrough;

void main()
{
 char (*Screen)[];
 (void*)Screen=MK_FP(0xB800,0);

 InstallKeyboardHandler(getpid());
 Int9PassThrough=0;
 do
 {
   for (int count=0; count<256; count++)
   {
     (*Screen)[count+count]=KeyMap[count]+'0';
   };
 }
 while (KeyMap[1]==0);

 //DeinstallKeyboardHandler(); not necessary because done automatically
};